;; -*- mode: sawfish; auto-compile-lisp: nil; -*-
;; Time-stamp: <01/03/2004 15:17:30 Yann Hodique>

(setq dl-load-path (cons "/home/yann/.sawfish/lib" dl-load-path))

;; Customization
(require 'sawmill-defaults)

;; Custom keymaps :
(defvar win-keymap
  "fast access keymap")

(defvar apps-keymap
  "apps keymap")

(defvar xmms-keymap
  "xmms control keymap")

(setq win-keymap (make-keymap))
(setq apps-keymap (make-keymap))
(setq xmms-keymap (make-keymap))

;; display grid when moving/resizing
(setq move-outline-mode 'box)
(setq resize-outline-mode 'box)

;; turn off bell
(system "xset b off")

;; define functions for interactive move/resize
(defun rsk-shove-size-window (dir inc &optional no-focus)
  "Size focused window by chunks 'left, 'right, 'up or 'down"
  (interactive)
  (let* ((win (input-focus))
         (dim (window-dimensions win))
         (dimx (car dim))
         (dimy (cdr dim)))
    (cond ((eq dir 'left) (setq dimx (- dimx inc)))
          ((eq dir 'right) (setq dimx (+ dimx inc)))
          ((eq dir 'up) (setq dimy (- dimy inc)))
          ((eq dir 'down) (setq dimy (+ dimy inc))))
    (resize-window-to win dimx dimy)
    (tab-refresh-group win 'resize)
    (unless no-focus
      (display-window win))))

(defun rsk-grid-place-window (grid inc)
  "move the window to the grid position specified by the numeric keypad"
  (interactive)
  (let* ((win (input-focus))
         (pos (window-position win))
         (posx (car pos))
         (posy (cdr pos))
         (x 0)
         (y 0))
    (cond ((or (equal grid 7) (equal grid 8) (equal grid 9)) 
           (setq y (- posy inc)))
          ((or (equal grid 4) (equal grid 5) (equal grid 6)) 
           (setq y posy))
          ((or (equal grid 1) (equal grid 2) (equal grid 3)) 
           (setq y (+ posy inc))))
    (cond ((or (equal grid 1) (equal grid 4) (equal grid 7)) 
           (setq x (- posx inc)))
          ((or (equal grid 2) (equal grid 5) (equal grid 8)) 
           (setq x posx))
          ((or (equal grid 3) (equal grid 6) (equal grid 9)) 
           (setq x (+ posx inc))))
    ;; the following is necessary because "/" function can result in
    ;; fractional numbers
    (move-window-to win x y)
    (tab-refresh-group win 'move)
    (display-window win)))

;; Dynamically define a shortcut for a window
(require 'sawfish.wm.util.keymap)
(defun define-shortcut (win)
  "Ask for a key and bind that key to select the current
 window. Will happily destroy previous binds for that key."
  (interactive "%w\ne")
  (when win
    (let ((key (read-event "Press shortcut key")))
      (bind-keys win-keymap key `(display-window ,win)))))

;; define apps shortcut
(defun def-app-key (key command)
  "define shortcut in apps-keymap for application"
  (bind-keys apps-keymap key `(run-shell-command ,command)))

;; Merlin stuff
(require 'merlin.pager)
(defpager pager)
(require 'contrib.wmresize)

(require 'merlin.menu)

(require 'merlin.fishbowl)
(deffishbowl fishbowl)

(require 'merlin.iconbox)
(deficonbox iconbox)
(require 'merlin.icons)

;; launch my apps
(defun init-desktop ()
  "Init my applications"
  (interactive)
  (system "korganizer &")
  (system "xterm &")
  (system "kopete &")
  )

;; Services config
(require 'services) 
(services-add "Google"  
           "mozilla-firefox -remote 'openurl(http://www.google.com/search?q=#)'"  
           services-clean-url-query-escaping) 
(services-add "Google: I'm Feeling Lucky"   
           "mozilla-firefox -remote 'openurl(http://www.google.com/search?q=#&sa=I%27m+Feeling+Lucky)'"   
           services-clean-url-query-escaping)  
(services-add "Mozilla-Firefox" "mozilla-firefox -remote 'openurl(#)'"  
           services-clean-url) 
;(services-add "Dictionary" "xterm -T 'dict: #' -e dict #") 
(services-add "Unix man" "xterm -g 160x50+200+150 -T 'man: #' -e sh -c 'man #; read'") 

;; Personal hack : fluxbox tabs
(require 'tab)

(defun mygroup (win)
  (interactive "%W")
  (if (marked-windows)
      (progn
        (apply-on-marked-windows (lambda (w) (tab-group-window w win)))
        (unmark-all-windows))
    (mark-window win)))
(require 'x-cycle)
(define-cycle-command-pair
  'cycle-tabgroup 'cycle-tabgroup-backwards
  (lambda (w)
    (delete-if-not window-in-cycle-p   
                   (delete-if (lambda (win)
                                (and (not (eq win w))
                                     (tab-same-group-p win w))) 
                              (workspace-windows current-workspace))
                   )
    )
  #:spec "%W")

(defun osd-message (msg)
  "Print message using osd"
  (system (format nil (_ "echo %s | osd_cat -s 5 -c green -f '-*-verdana-bold-r-normal-*-72-*' -p middle -A center -d 1 &") msg)))

;;; DisplayWorkspce (1.0)
;;; Martin Kavalec (martin@penguin.cz)
(defun display-ws-name ()
    (osd-message (or (nth current-workspace workspace-names)
        (format nil (_ "Workspace %d") (1+ current-workspace))))
    )
(add-hook 'enter-workspace-hook display-ws-name)

(require 'sawfish.wm.util.window-order)

(defun hide-list (windows) 
  (cond ((null windows) t) 
	(t 
         (mapcar iconify-window (delete-if (lambda (w) (window-get w 'never-focus)) windows))
	 )))

;(require 'merlin.ugliness)
(require 'send-to-head)
(require 'marks)
(require 'macros)

;; Rotate position/dimension of marked windows
(defun exchange-windows (clock)
  (interactive)
  (if (marked-windows)
      (let* ((windows-list (if (eq clock t)
                               (reverse marked-windows-list)
                             marked-windows-list))
             (win (nth (- (length windows-list) 1) windows-list))
             (dim (window-dimensions win))
             (pos (window-position win)))
        (let loop ((l windows-list))
             (if (null l)
                 nil
               (progn
                 (let* ((w (car l))
                        (dimtmp (window-dimensions w))
                        (postmp (window-position w)))
                   (move-resize-window-to w (car pos) (cdr pos) (car dim) (cdr dim))
                   (setq dim dimtmp
                         pos postmp))
                 (loop (cdr l)))
               )))))

;; Shortcuts

;; interactive resize
(bind-keys global-keymap "W-M-Left"      '(rsk-shove-size-window 'left 1))
(bind-keys global-keymap "W-M-Right"     '(rsk-shove-size-window 'right 1))
(bind-keys global-keymap "W-M-Up"        '(rsk-shove-size-window 'up 1))
(bind-keys global-keymap "W-M-Down"      '(rsk-shove-size-window 'down 1))
(bind-keys global-keymap "S-W-M-Left"    '(rsk-shove-size-window 'left 30))
(bind-keys global-keymap "S-W-M-Right"   '(rsk-shove-size-window 'right 30))
(bind-keys global-keymap "S-W-M-Up"      '(rsk-shove-size-window 'up 30))
(bind-keys global-keymap "S-W-M-Down"    '(rsk-shove-size-window 'down 30))

;; interactive move
(bind-keys global-keymap "W-M-j"    '(rsk-grid-place-window 1 1))
(bind-keys global-keymap "W-M-k"   '(rsk-grid-place-window 2 1))
(bind-keys global-keymap "W-M-l"   '(rsk-grid-place-window 3 1))
(bind-keys global-keymap "W-M-u"   '(rsk-grid-place-window 4 1))
(bind-keys global-keymap "W-M-i"  '(rsk-grid-place-window 5 1))
(bind-keys global-keymap "W-M-o"  '(rsk-grid-place-window 6 1))
(bind-keys global-keymap "W-M-7"   '(rsk-grid-place-window 7 1))
(bind-keys global-keymap "W-M-8"     '(rsk-grid-place-window 8 1))
(bind-keys global-keymap "W-M-9"  '(rsk-grid-place-window 9 1))

(bind-keys global-keymap "W-M-J"    '(rsk-grid-place-window 1 30))
(bind-keys global-keymap "W-M-K"   '(rsk-grid-place-window 2 30))
(bind-keys global-keymap "W-M-L"   '(rsk-grid-place-window 3 30))
(bind-keys global-keymap "W-M-U"   '(rsk-grid-place-window 4 30))
(bind-keys global-keymap "W-M-I"  '(rsk-grid-place-window 5 30))
(bind-keys global-keymap "W-M-O"  '(rsk-grid-place-window 6 30))
(bind-keys global-keymap "W-M-&"   '(rsk-grid-place-window 7 30))
(bind-keys global-keymap "W-M-*"     '(rsk-grid-place-window 8 30))
(bind-keys global-keymap "W-M-("  '(rsk-grid-place-window 9 30))

(bind-keys window-keymap "W-g" 'mygroup)
(bind-keys window-keymap "W-u" 'tab-release-window)
(bind-keys global-keymap "W-Left" 'tab-raise-left-window)
(bind-keys global-keymap "W-Right" 'tab-raise-right-window)
(bind-keys global-keymap "W-Tab" 'cycle-tabgroup)

(bind-keys window-keymap "W-d" 'define-shortcut)
(bind-keys global-keymap "W-w" 'win-keymap)
(bind-keys global-keymap "W-x" 'apps-keymap)

(bind-keys global-keymap "W-SPC" services)

(def-app-key "W-q" "qnet")
(def-app-key "W-e" "emacs")
(def-app-key "W-a" "aterm +sb -tr -fg white -tint green -sh 50 -fn fixed")
(def-app-key "W-x" "xterm")
(def-app-key "W-m" "mozilla-firefox")
(def-app-key "W-s" "emacs -f gnus")
(def-app-key "W-g" "gideon")
(def-app-key "W-p" "pvs")
(def-app-key "W-t" "xterm -g 160x50+200+150 -e 'cd ; devtodo -A ; read a'")

(bind-keys global-keymap "W-c" xmms-keymap)

(bind-keys xmms-keymap "W-p" (lambda () (system "if xmmsctrl playing; then xmmsctrl pause; else xmmsctrl play; fi")))
(bind-keys xmms-keymap "W-s" (lambda () (system "xmmsctrl stop")))
(bind-keys xmms-keymap "W-Left" (lambda () (system "xmmsctrl prev")))
(bind-keys xmms-keymap "W-Right" (lambda () (system "xmmsctrl next")))

(bind-keys global-keymap "W-a" (lambda () (hide-list (delete-if (lambda (w) (or (window-sticky-p w) (window-avoided-p w))) 
                                                                (cdr (window-order current-workspace))))))
(bind-keys global-keymap "W-s" (lambda () (mapcar (lambda (w) (uniconify-window w)) (managed-windows))))

(bind-keys window-keymap "W-o" 'send-to-next-head)
(bind-keys global-keymap "W-O" (lambda () (mapcar (lambda (w) (send-to-next-head w)) 
                                                  (delete-if (lambda (w) (or (window-sticky-p w) (window-avoided-p w))) 
                                                             (window-order current-workspace)))))

(bind-keys window-keymap "W-m" 'mark-window)
(bind-keys window-keymap "W-M" 'unmark-window)
(bind-keys global-keymap "W-BS" 'unmark-all-windows)

(bind-keys global-keymap "W-e" (lambda () (exchange-windows t)))
(bind-keys global-keymap "W-E" (lambda () (exchange-windows nil)))
