
;; DragScreen for SawMill
;;	rbowles@infonautics.com - Mon Dec 27 11:19:58 EST 1999
;;
;; disclaimers:
;;	Fuctions need to be documented.
;; 	This is the first Lisp I've coded in almost 5 years
;;		and it shows...
;;
(require 'smart-placement)
(require 'timers)

(defgroup dragscreen "Drag Screen")

(defcustom rb-drag-rendermode 'outline
	"Method of drawing windows during drag."
	:group dragscreen
	:type symbol
	:options (outline fancy opaque shade hybrid))

(defcustom rb-drag-speed 20
	"(10->1:1, 20->2:1, etc)"
	:group dragscreen
	:type number
	:range (10 . 100))

(defcustom rb-slide-indent 2
	"Pixels to indent while drawing a frame."
	:group dragscreen
	:type number
	:range (0 . 8))

(defcustom rb-drag-gridsnap 16
	"rb-drag-gridsnap"
	:group dragscreen
	:type number
	:range (0 . 32))

(defcustom rb-drag-edgesnap 32
	"Snap to screen edge if within this many pixels."
	:group dragscreen
	:type number
	:range (0 . 256))

(defcustom rb-slide-init-delay 250
	"Delay (in ms) before starting first drag."
	:group dragscreen
	:type number
	:range (0 . 1000))

(defcustom rb-slide-delay 50
	"Delay (in ms) after each drag operation."
	:group dragscreen
	:type number
	:range (0 . 500))

(defcustom rb-slide-message nil
	"Display infomation about what's being done."
	:group dragscreen
	:type boolean
)

(setq rb-drag-pre-ops (list
	(cons 'outline hide-window)
	(cons 'fancy hide-window)
	(cons 'opaque nop)
	(cons 'shade shade-window)
	(cons 'hybrid shade-window)
))

(setq rb-drag-post-ops (list
	(cons 'outline show-window)
	(cons 'fancy show-window)
	(cons 'opaque nop)
	(cons 'shade unshade-window)
	(cons 'hybrid unshade-window)
))

(defvar rb-slide-timer nil)
(defvar rb-ppos0 '())
(defvar assoc-win-list '())
(defvar outlines '())
(defvar rb-h 0)
(defvar rb-w 0)
(defvar max-x 0)
(defvar max-y 0)
(defvar rb-dragscreen-move-count 0)

;;-------------------

(defun rb-get-sliding ()
	(delete-if (lambda (w) (or
			(window-get w 'sticky)
			(window-get w 'iconified)))
		(managed-windows))
)

;; -- AWL (assoc-win-list) --
;; stores window information in an alist of alists of the general form:
;; <alist> -> ( (<win> . (<attrs>)) (<win> . (<attrs)) ... )
;; <attrs> -> ( ('pos . (x . y)) ('dim . (w . h)) ... )
;;
(defun rb-awl-init (&optional wlist)
	(setq assoc-win-list (mapcar (lambda (w) (let* (
				(offs (window-frame-offset w))
				(dc (window-dimensions w))
			)
			(list w '.
				`(pos . ,(window-position w))
				`(dim . ,(window-frame-dimensions w))
				`(name . ,(window-name w))
			)))
		(if (consp wlist) wlist (rb-get-sliding)))
	)
)

(defun assocset (key value alist) (let* ((temp (assoc key alist)))
	(if temp
		(rplaca (member temp alist) `(,key . ,value))
		(rplacd alist (append (cdr alist) `((,key . ,value))))
	)
))

(defun rb-awl-winop (func)
	(print func)
	(mapcar func (mapcar car assoc-win-list))
)

(defun rb-awl-set (w key value)
	(assocset key value (assoc w assoc-win-list))
)

(defun rb-awl-updatexy ()
	(mapcar (lambda (al) (let* (
			(p (window-position (car al)))
		)
		(assocset 'pos p al)
		)
	) assoc-win-list)
)

;; assoc-win-list drawing functions
(defun rb-awl-drawall (func)
	(mapcar (lambda (al) (let* (
			(pos (cdr (assoc 'pos al)))
			(dim (cdr (assoc 'dim al)))
			(x (car pos))
			(y (cdr pos))
			(w (car dim))
			(h (cdr dim))
		) (cond
			((eq rb-drag-rendermode 'outline)
				(func 'box
					(+ x rb-slide-indent)
					(+ y rb-slide-indent)
					(- w (* 2 rb-slide-indent))
					(- h (* 2 rb-slide-indent)))

			t)
			((eq rb-drag-rendermode 'hybrid)
				(func 'box
					(+ x rb-slide-indent)
					(+ y rb-slide-indent 18)
					(- w (* 2 rb-slide-indent))
					(- h (* 2 rb-slide-indent) 18))

			t)
			((eq rb-drag-rendermode 'fancy)
				(func 'box
					(+ x rb-slide-indent)
					(+ y rb-slide-indent)
					(- w (* 2 rb-slide-indent))
					(- h (* 2 rb-slide-indent)))
				(func 'box
					(+ x rb-slide-indent 3)
					(+ y rb-slide-indent 3)
					(- w (* 2 rb-slide-indent) 6)
					12) ;; (- h (* 2 rb-slide-indent) 18)
			)
		)
	)) assoc-win-list )
)

(defun rb-draw-outlines () (rb-awl-drawall draw-window-outline))
(defun rb-erase-outlines () (rb-awl-drawall erase-window-outline))

;; rb-constrain
;;	applies 'rb-drag-edgesnap' and 'rb-drag-gridsnap'
;;	returns value in given range
;;
(defun rb-constrain (i l-min l-max snapto)
	(setq imod (% (+ i rb-drag-edgesnap) snapto))
	(if (< imod (* 2 rb-drag-edgesnap))
		(setq i (+ (- i imod) rb-drag-edgesnap)))
	(if (> i l-max) (setq i l-max))
	(if (< i l-min) (setq i l-min))
	(setq i (- i (% i rb-drag-gridsnap)))
	i
)

;; -- dragscreen code -- ;;
(defun rb-dragscreen-init ()
	;; (grab-pointer)
	(setq rb-w (screen-width))
	(setq rb-h (screen-height))
	(setq max-x (* rb-w (- viewport-columns 1)))
	(setq max-y (* rb-h (- viewport-rows 1)))
	(setq rb-x0v viewport-x-offset)
	(setq rb-y0v viewport-x-offset)
	(setq rb-dx-last viewport-x-offset)
	(setq rb-dy-last viewport-x-offset)
	(setq rb-ppos0 (query-pointer))
	(setq rb-dragscreen-move-count 0)
	(rb-awl-winop (cdr (assoc rb-drag-rendermode rb-drag-pre-ops)))
)

(defun rb-dragscreen-move ()
	"Slide Stuff Around"
	(interactive)
	(let* (
		(rb-ppos1 (query-pointer))
		(dx (- (car rb-ppos0) (car rb-ppos1)))
		(dy (- (cdr rb-ppos0) (cdr rb-ppos1)))
		(rb-dx (rb-constrain (+ rb-x0v (/ (* dx rb-drag-speed) 10))
			0 max-x rb-w))
		(rb-dy (rb-constrain (+ rb-y0v (/ (* dy rb-drag-speed) 10))
			0 max-y rb-h))
	) (unless (or
			rb-slide-timer
			(and (eql rb-dx rb-dx-last) (eql rb-dy rb-dy-last))
		)
		(setq rb-slide-timer (make-timer (lambda ()
				(setq rb-slide-timer nil)
			) 0 rb-slide-delay))
		(setq rb-dragscreen-move-count (+ 1 rb-dragscreen-move-count))
		(if rb-slide-message
			(show-message (format nil "offs=(%d+%d, %d+%d)"
				(/ rb-dx rb-w) (% rb-dx rb-w)
				(/ rb-dy rb-h) (% rb-dy rb-h)))
		)
		(rb-erase-outlines)
		(set-viewport rb-dx rb-dy)
		(rb-awl-updatexy)
		(rb-draw-outlines)
		(setq rb-dx-last rb-dx)
		(setq rb-dy-last rb-dy)
	)) ;; end unless+let
)

(defun rb-dragscreen-cleanup ()
	"Stop the Madness"
	(interactive)
	(print "start rb-dragscreen-cleanup")
	(show-message nil)
	(cond
	((consp rb-ppos0)
		(rb-erase-outlines)
		(sync-server)
		(rb-awl-winop (cdr (assoc rb-drag-rendermode rb-drag-post-ops)))
		(setq rb-ppos0 ())
		t)
	)
	(ungrab-pointer)
	(ungrab-keyboard)
	(when rb-slide-message
		(show-message "sliding done")
		(setq rb-slide-timer (make-timer (lambda ()
				(setq rb-slide-timer nil)
				(show-message nil)
			) 0 500 ))
	)
	(print "end rb-dragscreen-cleanup")
)

(defun rb-dragscreen-done ()
	(interactive)
	(print rb-dragscreen-move-count)
	(print "rb-dragscreen-done")
	(throw 'slide-done)
)

(defvar rb-drag-keyap (bind-keys (make-keymap)
	"Any-Off" 'rb-dragscreen-done
	"Any-ESC" 'rb-dragscreen-done
	"Any-Move" 'rb-dragscreen-move)
)

(defun rb-dragscreen-main ()
	(print "start rb-dragscreen-main")
	(grab-pointer)
	(grab-keyboard)
	(sync-server)
	(rb-draw-outlines)
	(let* (
		(unbound-key-hook nil)
		(disable-auto-raise t)
		(override-keymap rb-drag-keyap)
		(from-motion-event (and
			(current-event)
			(string-match "-Move$" (event-name (current-event)))
		))
	) (unwind-protect (progn
		(catch 'slide-done
			(when from-motion-event (rb-dragscreen-move))
			(recursive-edit))
		(ungrab-keyboard)
		(ungrab-pointer)
		(rb-dragscreen-cleanup)
	)))
	(print "end rb-dragscreen-main")
	(print "...")
)

(defun rb-dragscreen ()
	"Start the Madness"
	(interactive)

	(print "start rb-dragscreen")
	(rb-awl-init)
	(rb-dragscreen-init)
	;;
	;; Ouch! some apps (xawtv) take a while to hide
	;; and interfere with draws...
	(setq rb-slide-timer (make-timer
		(lambda () (unwind-protect (progn
			(setq rb-slide-timer nil)
			(rb-dragscreen-main)
		)))
		(/ rb-slide-init-delay 1000)
		(% rb-slide-init-delay 1000)
	))
	(print "end rb-dragscreen")
)

(provide 'dragscreen)
