;; frame.jl - Provide splitable non-overlapping groups (inspired by ion/ratpoison/wmi/...)
;;
;; Author : Yann Hodique <Yann.Hodique@lifl.fr>

;; Structure of a frame:
;; frame ::= (group-offset frame-offset) -- group / parent
;;           | ('horiz frame-offset size frame-offset frame-offset) -- dir / parent / ratio / frame1 / frame2
;;           | ('vert frame-offset size frame-offset frame-offset) -- dir / parent / ratio / frame1 / frame2

(define-structure sigma.frame

    (export)

    (open sigma.group)

  (define-record-type :sigframe
    (sigframe type #!optional parent #!key (ratio 0.5) (first nil) (second nil))
    sigframe-p
    (type sigframe-type set-sigframe-type)
    (parent sigframe-parent set-sigframe-parent)
    (ratio sigframe-ratio set-sigframe-ratio)
    (first sigframe-first set-sigframe-first)
    (second sigframe-second set-sigframe-second))

  (define-record-discloser :sigframe (lambda (x) `(sigframe ,(sigframe-type x) ,(sigframe-parent x))))

  (define sigma-frame-frames nil)

  ;; Returns frame object corresponding to frameptr
  (define (sigma-frame-deref frameptr)
    (nth frameptr sigma-frame-frames))

  ;; Returns true if frame is an elementary frame
  (define (sigma-frame-is-simple-frame frameptr)
    (let ((type (sigframe-type (sigma-frame-deref frameptr))))
      (not (or (eq type 'horiz) (eq type 'vert)))))

  ;; Moves/resizes a frame
  (define (sigma-frame-move-resize-frame-to frameptr x y w h)

    ;; todo: code :-)

    )

  ;; Splits the first elementary subframe if it contains at least 2 windows
  (define (sigma-frame-split-frame frameptr dir)
    (if (sigma-frame-is-simple-frame frameptr)
        (if (< 1 (length (tab-group-window-list (nth (sigframe-type (sigma-frame-deref frameptr)) tab-groups))))
            (let* ((fr (sigma-frame-deref frameptr))
                   (nb (length sigma-frame-frames))
                   first
                   second)
              (rplaca (nthcdr frame sigma-frame-frames)
                      (sigframe dir (sigframe-parent fr) #:first nb #:second (+ nb 1)))

              ;; todo: build first & second

              (setq sigma-frame-frames (append sigma-frame-frames (list first second)))))
      (sigma-frame-split-frame (sigframe-first (sigma-frame-deref frameptr)) dir)))

  ;; Merges the 2 components of a frame
  (define (sigma-frame-join-frame frameptr)
    (if (not (sigma-frame-is-simple-frame frameptr))

        ;; todo: code :-)

        ))

  ;; Returns the parent frame index, or nil if root frame
  (define (sigma-frame-parent-frame frameptr)
    (sigframe-parent (sigma-frame-deref frameptr)))

  ;; Detaches a window from its group, returns the window object
  (define (sigma-frame-detach-window win)
    (tab-delete-window-from-tab-groups win)
    win)

  ;; Attaches a window to the group in the first elementary subframe
  (define (sigma-frame-attach-window win frameptr)
    (let ((fr (sigma-frame-deref frameptr)))
      (if (sigma-frame-is-simple-frame frameptr)
          (tab-put-window-in-group win (sigframe-type fr))
        (sigma-frame-attach-window win (sigframe-first fr)))))

  ;; Returns a pointer the inner frame containing a window
  (define (sigma-frame-window-frame win)
    (let ((gindex (tab-window-group-index win)))
      (let loop ((frames sigma-frame-frames)
                 (findex 0))
           (cond ((null frames) nil)
                 ((and (sigma-frame-is-simple-frame findex)
                       (equal gindex (sigframe-type (car frames))))
                  findex)
                 (t (loop (cdr frames) (+ findex 1)))))))

  )
