;; refs.jl - Store references tables
;;
;; Author : Yann Hodique <Yann.Hodique@lifl.fr>

(define-structure sigma.refs

    (export sigtable-p
            sigma-make-table
            sigma-table-ref
            sigma-table-bound-p
            sigma-table-set
            sigma-table-unset
            sigma-table-walk
            sigma-table-size)

    (open rep
          rep.data.records
          rep.data.tables)

  (define-record-type :sigtable
    (sigtable hash next)
    sigtable-p
    (hash sigtable-hash)
    (next sigtable-next-ref sigtable-set-next-ref)
    (free sigtable-free sigtable-set-free))

  (define-record-discloser :sigtable (lambda (x) `(sigtable ,(sigtable-hash x) ,(sigtable-next-ref x) ,(sigtable-free x))))

  (define (sigma-make-table)
    (sigtable (make-table eq-hash eq) 0 nil))

  (define (sigma-table-ref table key)
    (table-ref (sigtable-hash table) key))

  (define (sigma-table-bound-p table key)
    (table-bound-p (sigtable-hash table) key))

  (define (sigma-table-set table value #!optional key)
    (if (or (null key)
            (not (numberp key))
            (> key (sigtable-next-ref table)))
        (if (sigtable-free table)
            (progn
              (setq key (car (sigtable-free table)))
              (sigtable-set-free table (remq key (sigtable-free table))))
          (progn
            (setq key (sigtable-next-ref table))
            (sigtable-set-next-ref table (+ key 1)))))
    (table-set (sigtable-hash table) key value)
    key)

  (define (sigma-table-unset table key)
    (if (equal key (- (sigtable-next-ref table) 1))
        (sigtable-set-next-ref table key)
      (sigtable-set-free table (append (sigtable-free table) (list key))))
    (table-unset (sigtable-hash table) key))

  (define (sigma-table-walk function table)
    (table-walk function (sigtable-hash table)))

  (define (sigma-table-size table)
    (sigtable-next-ref table))

  )
