;; tab.jl - Emulate fluxbox tabs system
;;
;; Author : Yann Hodique <Yann.Hodique@lifl.fr>
;;
;; Usage:
;;  Copy this file to somewhere in your sawfish load-path, for example
;;  "~/.sawfish/lisp". Then add (require 'tab) to your ~/.sawfishrc and restart
;;  sawfish or issue the (load "tab") from sawfish-client.
;;
;; You can use something like this as shortcuts (put this after requirement):
;; (bind-keys window-keymap "W-g" 'tab-group-window)
;; (bind-keys window-keymap "W-u" 'tab-release-window)
;; (bind-keys window-keymap "W-Left" 'tab-raise-left-window)
;; (bind-keys window-keymap "W-Right" 'tab-raise-right-window)
;; (require 'x-cycle)
;; (define-cycle-command-pair
;;   'cycle-tabgroup 'cycle-tabgroup-backwards
;;   (lambda (w)
;;     (delete-if-not window-in-cycle-p
;;                    (delete-if (lambda (win)
;;                                 (and (not (eq win w))
;;                                      (tab-same-group-p win w)))
;;                               (workspace-windows current-workspace))
;;                    )
;;     )
;;   #:spec "%W")
;; (bind-keys window-keymap "W-Tab" 'cycle-tabgroup)

;; Note : to display the tab frame, you will need a modified theme

(define-structure sigma.tab

    (export sigma-tab-window-group-left-edge
            sigma-tab-window-group-width)

    (open rep
          custom
          sawfish.wm.frames
          sigma.group)

  (defgroup tabs "Tabs")

  (defcustom tab-default-width 60 "Default tab width"
    :group tabs
    :type number)

  (defcustom tab-default-interval 4 "Default tab interval"
    :group tabs
    :type number)

  ;; Compute width of tab for group
  (define (sigma-tab-group-tab-width group)
    (let ((w (car (siggroup-dimensions group)))
          (l (length (siggroup-window-list group))))
      (if (> (* l tab-default-width) w)
          (quotient w l)
        tab-default-width)))

  ;; Compute left edge of tab
  (define (sigma-tab-window-group-left-edge win)
    (let* ((gr (sigma-group-find-window win))
           (r (sigma-group-rank win (siggroup-window-list gr))))
      (* r (+ (sigma-tab-group-tab-width gr) tab-default-interval))
      ))

  ;; Compute left edge of tab
  (define (sigma-tab-window-group-width win)
    (sigma-tab-group-tab-width (sigma-group-find-window win)))

  ;; new class : tab
  (def-frame-class tab
    `((cursor . left_ptr)
      (x-justify . center)
      (y-justify . center)
      (left-edge . ,sigma-tab-window-group-left-edge)
      (width . ,sigma-tab-window-group-width)
      ))

  )
