
(defcustom viewport-scroll-steps 64
	"Number of steps to use when scrolling between viewports"
	:group workspace
	:type number
	:range (1 . 256))

;; returns t if it actually moved the viewport
(defun scroll-screen-viewport (col row)
  (when (and (>= col 0) (< col viewport-columns)
	     (>= row 0) (< row viewport-rows))
	(setq x-jump 
		(/ (- (* col (screen-width)) viewport-x-offset) 
		viewport-scroll-steps))
	(setq y-jump 
		(/ (- (* row (screen-height)) viewport-y-offset) 
		viewport-scroll-steps))
	(setq n-jumps viewport-scroll-steps)
	(while (> n-jumps 0)
		(set-viewport (+ viewport-x-offset x-jump)
			      (+ viewport-y-offset y-jump))
		(setq n-jumps (- n-jumps 1)))
	(set-viewport (* col (screen-width)) (* row (screen-height)))
    t))

;; returns t if it actually moved the viewport
(defun scroll-viewport (right down)
  (let
      ((port (screen-viewport)))
    (scroll-screen-viewport (+ (car port) right)
			 (+ (cdr port) down))))

(defun scroll-viewport-right ()
  "Scroll the viewport one screen to the right."
  (interactive)
  (scroll-viewport 1 0))

(defun scroll-viewport-left ()
  "Scroll the viewport one screen to the left."
  (interactive)
  (scroll-viewport -1 0))

(defun scroll-viewport-down ()
  "Scroll the viewport one screen down."
  (interactive)
  (scroll-viewport 0 1))

(defun scroll-viewport-up ()
  "Scroll the viewport one screen up."
  (interactive)
  (scroll-viewport 0 -1))

